@@ -36,6 +36,7 @@ from .cyclonedx import CycloneDx
 from .spdxlite import SpdxLite
 from .csvoutput import CsvOutput
 from . import __version__
+from .scanner import FAST_WINNOWING
 
 
 def print_stderr(*args, **kwargs):
@@ -49,7 +50,7 @@ def setup_args() -> None:
     """
     Setup all the command line arguments for processing
     """
-    parser = argparse.ArgumentParser(description=f'SCANOSS Python CLI. Ver: {__version__}, License: MIT')
+    parser = argparse.ArgumentParser(description=f'SCANOSS Python CLI. Ver: {__version__}, License: MIT, Fast Winnowing: {FAST_WINNOWING}')
     parser.add_argument('--version', '-v', action='store_true', help='Display version details')
 
     subparsers = parser.add_subparsers(title='Sub Commands', dest='subparser', description='valid subcommands',
@@ -59,6 +60,12 @@ def setup_args() -> None:
     p_ver = subparsers.add_parser('version', aliases=['ver'],
                                   description=f'Version of SCANOSS CLI: {__version__}', help='SCANOSS version')
     p_ver.set_defaults(func=ver)
+
+    # Sub-command: fast
+    p_ver = subparsers.add_parser('fast',
+                                  description=f'Is fast winnowing enabled: {__version__}', help='SCANOSS fast winnowing')
+    p_ver.set_defaults(func=fast)
+
     # Sub-command: scan
     p_scan = subparsers.add_parser('scan', aliases=['sc'],
                                    description=f'Analyse/scan the given source base: {__version__}',
@@ -118,6 +125,9 @@ def setup_args() -> None:
     p_wfp.add_argument('--output', '-o', type=str, help='Output result file name (optional - default stdout).')
     p_wfp.add_argument('--obfuscate', action='store_true', help='Obfuscate fingerprints')
     p_wfp.add_argument('--skip-snippets', '-S', action='store_true', help='Skip the generation of snippets')
+    p_wfp.add_argument('--all-extensions', action='store_true', help='Fingerprint all file extensions')
+    p_wfp.add_argument('--all-folders', action='store_true', help='Fingerprint all folders')
+    p_wfp.add_argument('--all-hidden', action='store_true', help='Fingerprint all hidden files/folders')
 
     # Sub-command: dependency
     p_dep = subparsers.add_parser('dependencies', aliases=['dp', 'dep'],
@@ -232,7 +242,6 @@ def setup_args() -> None:
         exit(1)
     args.func(parser, args)  # Execute the function associated with the sub-command
 
-
 def ver(*_):
     """
     Run the "ver" sub-command
@@ -240,6 +249,12 @@ def ver(*_):
     """
     print(f'Version: {__version__}')
 
+def fast(*_):
+    """
+    Run the "fast" sub-command
+    :param _: ignored/unused
+    """
+    print(f'Fast Winnowing: {FAST_WINNOWING}')
 
 def file_count(parser, args):
     """
@@ -293,7 +308,9 @@ def wfp(parser, args):
         open(scan_output, 'w').close()
 
     scan_options = 0 if args.skip_snippets else ScanType.SCAN_SNIPPETS.value  # Skip snippet generation or not
-    scanner = Scanner(debug=args.debug, quiet=args.quiet, obfuscate=args.obfuscate, scan_options=scan_options)
+    scanner = Scanner(debug=args.debug, trace=args.trace, quiet=args.quiet, obfuscate=args.obfuscate,
+                      scan_options=scan_options, all_extensions=args.all_extensions,
+                      all_folders=args.all_folders, hidden_files_folders=args.all_hidden)
 
     if not os.path.exists(args.scan_dir):
         print_stderr(f'Error: File or folder specified does not exist: {args.scan_dir}.')
@@ -544,43 +561,40 @@ def utils_cert_download(_, args):
     :param _: ignore/unused
     :param args: Parsed arguments
     """
-    import ssl
     from urllib.parse import urlparse
     import socket
+    from OpenSSL import SSL, crypto
     import traceback
 
     file = sys.stdout
     hostname = 'unset'
     port = 'unkown'
+    if args.output:
+        file = open(args.output, 'w')
+    parsed_url = urlparse(args.hostname)
+    hostname = parsed_url.hostname or args.hostname  # Use the parse hostname, or it None use the supplied one
+    port = int(parsed_url.port or args.port)  # Use the parsed port, if not use the supplied one (default 443)
+    certs = []
     try:
-        if args.output:
-            file = open(args.output, 'w')
-        parsed_url = urlparse(args.hostname)
-        hostname = parsed_url.hostname or args.hostname  # Use the parse hostname, or it None use the supplied one
-        port = int(parsed_url.port or args.port)  # Use the parsed port, if not use the supplied one (default 443)
-        conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-        sock = context.wrap_socket(conn, server_hostname=hostname)
-        if not args.quiet or args.debug:
-            print_stderr(f'Attempting to download PEM certificate from {hostname}:{port} ...')
         if args.debug:
-            print_stderr('Connecting to host...')
-        sock.connect((hostname, port))
-        if args.debug:
-            print_stderr('Getting peer cert...')
-        peer_cert = sock.getpeercert(True)
-        if not peer_cert:
-            print_stderr(f'Error: Failed to download peer certificate data from {hostname}:{port}')
-            exit(1)
-        if args.debug:
-            print_stderr('Converting DER to PEM...')
-        cert_data = ssl.DER_cert_to_PEM_cert(peer_cert)
-        if not cert_data or cert_data == '':
-            print_stderr(f'Error: Failed to convert certificate data to PEM from {hostname}:{port}')
-            exit(1)
-        else:
-            print(cert_data.strip(), file=file)  # Print the downloaded PEM certificate
-    except Exception as e:
+            print_stderr(f'Connecting to {hostname} on {port}...')
+        conn = SSL.Connection(SSL.Context(SSL.TLSv1_2_METHOD), socket.socket())
+        conn.connect((hostname, port))
+        conn.do_handshake()
+        certs = conn.get_peer_cert_chain()
+        for index, cert in enumerate(certs):
+            cert_components = dict(cert.get_subject().get_components())
+            if(sys.version_info[0] >= 3):
+                cn = cert_components.get(b'CN')
+            else:
+                cn = cert_components.get('CN')
+            if not args.quiet:
+                print_stderr(f'Centificate {index} - CN: {cn}')
+            if(sys.version_info[0] >= 3):
+                print((crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode('utf-8')).strip(), file=file)  # Print the downloaded PEM certificate
+            else:
+                print((crypto.dump_certificate(crypto.FILETYPE_PEM, cert)).strip(), file=file)
+    except SSL.Error as e:
         print_stderr(f'ERROR: Exception ({e.__class__.__name__}) Downloading certificate from {hostname}:{port} - {e}.')
         if args.debug:
             traceback.print_exc()
